// ===========================================================================
//	IUGSTAS.h   			 Copyright R.W. Le Maitre 2002 - Version 1.0
// ===========================================================================

#ifndef IUGSTASH
#define IUGSTASH

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include <string.h>
#include <math.h>

// Atomic weights from which all molecular weights are calculated.
const	double		atWt_Si			= 28.086;
const	double		atWt_Ti			= 47.9;
const	double		atWt_Al			= 26.98154;
const	double		atWt_Fe			= 55.847;
const	double		atWt_Mn			= 54.938;
const	double		atWt_Mg			= 24.305;
const	double		atWt_Ca			= 40.08;
const	double		atWt_Na			= 22.98977;
const	double		atWt_K			= 39.098;
const	double		atWt_P			= 30.97376;
const	double		atWt_H			= 1.0079;
const	double		atWt_C			= 12.011;
const	double		atWt_Zr			= 91.22;
const	double		atWt_Cr			= 51.996;
const	double		atWt_V			= 50.9414;
const	double		atWt_Ni			= 58.71;
const	double		atWt_Co			= 58.9332;
const	double		atWt_Ba			= 137.34;
const	double		atWt_Sr			= 87.62;
const	double		atWt_Rb			= 85.4678;
const	double		atWt_F			= 18.9984;
const	double		atWt_Cl			= 35.4530;
const	double		atWt_S			= 32.06;
const	double		atWt_O			= 15.9994;
	
// Oxygen equivalents
const	double		oxyEquiv_F		= (0.5 * atWt_O / atWt_F);
const	double		oxyEquiv_Cl		= (0.5 * atWt_O / atWt_Cl);
const	double		oxyEquiv_S		= (0.5 * atWt_O / atWt_S);

// Molecular weights on basis of 1 cation
const	double		molWt_SiO2		= (atWt_Si + atWt_O * 2);
const	double		molWt_TiO2		= (atWt_Ti + atWt_O * 2);
const	double		molWt_Al2O3		= (atWt_Al + atWt_O * 1.5);
const	double		molWt_Fe2O3		= (atWt_Fe + atWt_O * 1.5);
const	double		molWt_FeO		= (atWt_Fe + atWt_O);
const	double		molWt_MnO		= (atWt_Mn + atWt_O);
const	double		molWt_MgO		= (atWt_Mg + atWt_O);
const	double		molWt_CaO		= (atWt_Ca + atWt_O);
const	double		molWt_Na2O		= (atWt_Na + atWt_O * 0.5);
const	double		molWt_K2O		= (atWt_K + atWt_O * 0.5);
const	double		molWt_P2O5		= (atWt_P + atWt_O * 2.5);
const	double		molWt_H2OPlus	= (atWt_H + atWt_O * 0.5);
const	double		molWt_CO2		= (atWt_C + atWt_O * 2);
const	double		molWt_ZrO2		= (atWt_Zr + atWt_O * 2);
const	double		molWt_Cr2O3		= (atWt_Cr + atWt_O * 1.5);
const	double		molWt_V2O3		= (atWt_V+ atWt_O * 1.5);
const	double		molWt_NiO		= (atWt_Ni + atWt_O);
const	double		molWt_CoO		= (atWt_Co + atWt_O);
const	double		molWt_BaO		= (atWt_Ba + atWt_O);
const	double		molWt_SrO		= (atWt_Sr + atWt_O);
const	double		molWt_Rb2O		= (atWt_Rb + atWt_O * 0.5);
const	double		molWt_F			= (atWt_F);
const	double		molWt_Cl		= (atWt_Cl);
const	double		molWt_S			= (atWt_S);

// Normative molecular weights 
const	double		molWt_Q			= (molWt_SiO2);
const	double		molWt_C			= (molWt_Al2O3);
const	double		molWt_Z			= (molWt_ZrO2 + molWt_SiO2);
const	double		molWt_or		= (molWt_K2O + molWt_Al2O3 + molWt_SiO2 * 3);
const	double		molWt_ab		= (molWt_Na2O + molWt_Al2O3 + molWt_SiO2 * 3);
const	double		molWt_an		= (molWt_CaO + (molWt_Al2O3 + molWt_SiO2) * 2);
const	double		molWt_ne		= (molWt_Na2O + molWt_Al2O3 + molWt_SiO2);
const	double		molWt_lc		= (molWt_K2O + molWt_Al2O3 + molWt_SiO2 * 2);
const	double		molWt_kp		= (molWt_K2O + molWt_Al2O3 + molWt_SiO2);
const	double		molWt_hl		= (atWt_Na + atWt_Cl);
const	double		molWt_nc		= (molWt_Na2O * 2 + molWt_CO2);
const	double		molWt_ac		= (molWt_Na2O + molWt_Fe2O3 + molWt_SiO2 * 2);
const	double		molWt_ns		= (molWt_Na2O * 2 + molWt_SiO2);
const	double		molWt_ks		= (molWt_K2O * 2 + molWt_SiO2);
const	double		molWt_di		= (molWt_CaO + molWt_MgO + molWt_SiO2 * 2);
const	double		molWt_hy		= (molWt_MgO + molWt_SiO2);
const	double		molWt_wo		= (molWt_CaO + molWt_SiO2);
const	double		molWt_ol		= (molWt_MgO * 2 + molWt_SiO2);
const	double		molWt_cs		= (molWt_CaO * 2 + molWt_SiO2);
const	double		molWt_cm		= (molWt_FeO + molWt_Cr2O3 * 2);
const	double		molWt_hm		= (molWt_Fe2O3);
const	double		molWt_mt		= (molWt_FeO + molWt_Fe2O3 * 2);
const	double		molWt_il		= (molWt_FeO + molWt_TiO2);
const	double		molWt_tn		= (molWt_CaO + molWt_TiO2 + molWt_SiO2);
const	double		molWt_pf		= (molWt_CaO + molWt_TiO2);
const	double		molWt_ru		= (molWt_TiO2);
const	double		molWt_ap		= (molWt_CaO * 5 + molWt_P2O5 * 3 + atWt_F * (1.0 - oxyEquiv_F));
const	double		molWt_hap		= (molWt_CaO * 5 + molWt_P2O5 * 3 + molWt_H2OPlus);
const	double		molWt_fl		= (atWt_Ca + atWt_F * 2);
const	double		molWt_py		= (atWt_Fe + atWt_S * 2);
const	double		molWt_cc		= (molWt_CaO + molWt_CO2);
const	double		molWt_mag		= (molWt_MgO + molWt_CO2);
const	double		molWt_sid		= (molWt_FeO + molWt_CO2);
const	double		molWt_h2p		= (molWt_H2OPlus);
const	double		molWt_fsd		= (molWt_SiO2);

// Correction factors for various substitutions
const	double		Rb_K_CorFac		= (molWt_Rb2O - molWt_K2O);
const	double		Sr_Ca_CorFac	= (molWt_SrO - molWt_CaO);
const	double		Ba_Ca_CorFac	= (molWt_BaO - molWt_CaO)  ;
const	double		V_Fe3_CorFac	= (molWt_V2O3 - molWt_Fe2O3);
const	double		Co_Fe2_CorFac	= (molWt_CoO - molWt_FeO);
const	double		Ni_Fe2_CorFac	= (molWt_NiO - molWt_FeO);
const	double		Mn_Fe2_CorFac	= (molWt_MnO - molWt_FeO);
const	double		Fe2_Mg_CorFac	= (molWt_FeO - molWt_MgO);

const	double		FeFac  		    = molWt_FeO / molWt_Fe2O3;

const	char		tabChar 		= '\t';
const	char		bellChar		= '\a';
const	char		sixAst[9] 		= "******  ";
const	char		warnMess[11]	= "WARNING - ";
const	char		warnSpaces[11]	= "          ";
//const	char		badData[5]		= "????";

typedef struct	
{	double m;
	double c;
}SLine;

typedef enum {
	kCO2AsOthers = 1,
	kCO2AsNaCarb,
	kCO2AsCaMgFeCarb
} ECO2Usage;

typedef enum {
	kWaterCO2Free = 1,
	kWithWaterCO2,
	kDontRecalculate
} ERecalculate;

typedef enum {
	kOK	= 1,
	kEndOfLine,
	kEndOfFile,
	kDataTruncated
}EReadStatus;

typedef struct {
	char	name[7];
} CStr6;

const	int		numOutputFiles	= 6;

const	int		numTableCols	= 10;

const	int		numOxides		= 29;
const	int		numNormValues	= 46;

const	int		lenFileName 	= 256;

const	int		lenSpecName		= 256;
const	int		lenOptName 		= 32;
const	int		lenRootName 	= 128;

const	int		lenTextInput	= 256;

const	double	reportEveryNth	= 200.0;
 
typedef struct {
	char	name[lenFileName];
} SFileName;

typedef struct {
	char	name[lenSpecName];
} SSpecimenName;

const CStr6	oNames[numOxides] = {{"SiO2"}, {"TiO2"}, {"Al2O3"}, {"Fe2O3"}, {"FeO"}, {"MnO"}, {"MgO"}, {"CaO"}, {"Na2O"}, 
			{"K2O"}, {"P2O5"}, {"H2O+"}, {"H2O-"}, {"CO2"}, {"Other"}, {"ZrO2"}, {"Cr2O3"}, {"V2O3"}, 
			{"NiO"}, {"CoO"}, {"BaO"}, {"SrO"}, {"Rb2O"}, {"F"}, {"Cl"}, {"S"}, {"Total"}, {"Ox.Eq"}, {"TAlk"}};
					
typedef struct {
	double	SiO2, TiO2, Al2O3, Fe2O3, FeO, MnO, MgO, CaO, Na2O, K2O,				// 0 - 9
			P2O5, H2OPlus, H2OMinus, CO2, Other, ZrO2, Cr2O3, V2O3, NiO, CoO, 		// 10 - 19
			BaO, SrO, Rb2O, F, Cl, S, Total, OxEq, TAlk;							// 20 - 28
}SOxideNames;

const CStr6	nNames[numNormValues] = {{"Q"}, {"C"}, {"Z"}, {"or"}, {"ab"}, {"an"}, {"ne"}, {"lc"}, {"kp"}, {"hl"}, 
			{"nc"}, {"ac"}, {"ns"}, {"ks"}, {"di"}, {"hy"}, {"wo"}, {"ol"}, {"cs"}, {"cm"}, 
			{"hm"}, {"mt"}, {"il"}, {"tn"}, {"pf"}, {"ru"}, {"ap"}, {"hap"}, {"fr"}, {"pr"}, 
			{"cc"}, {"mag"}, {"sid"}, {"H2O+"}, {"H2O-"}, {"Other"}, {"Fsd"}, {"P2O5+"}, {"CO2+"}, {"Cr2O3+"},
			{"F+"}, {"Cl+"}, {"S+"}, {"Total"}, {"MgFe"}, {"SiUnd"}};

typedef struct {
	double	Q, C, Z, orth, ab, an, ne, lc, kp, hl, 									// 0 - 9
			nc, ac, ns, ks, di, hy, wo, ol, cs, cm, 								// 10 - 19
			hm, mt, il, tn, pf, ru, ap, hap, fr, pr, 								// 20 - 29
			cc, mag, sid, H2OPlus, H2OMinus, Other, Fsd, exP2O5, exCO2, exCr2O3,	// 30 - 39
			exF ,exCl, exS, Total, MgRatio, SiUnd;									// 40 - 45
}SNormNames;

typedef union {
		SOxideNames		named;					// oxides by name
		double			wt[numOxides];			// oxide array
}SOxidepc;

typedef union {
		SNormNames  	named;					// normative minerals by name
		double			wt[numNormValues];		// normative mineral array
}SNormpc;

typedef struct {
		SSpecimenName	specimen;
		SOxidepc		oxide;
		SNormpc			norm;
}SAnalysisData;

class IUGSTAS {
	public:
						IUGSTAS();
						
		virtual			~IUGSTAS();

		virtual void		Init();
		
		virtual bool		AskForInputFile(char* taskName = "");
		
		virtual bool		OpenInputFile(char *fName);
		virtual void		CloseInputFile();
				
		virtual bool		AskForOutputTables(char* taskName = "");
		virtual bool		OpenOutputFile(char *fName, int nos);
		virtual void		CloseOutputFile(int nos);
		virtual void		CloseAllOutputFiles();

		virtual void		ReadTabDelimitedRow();
		virtual bool		EndOfFile();

		virtual int			IsValidOxide(char *inStr);
		virtual bool		StringsEqual(char *inStr1, char * inStr2);

		virtual void		GetOxidesInterActively();
		virtual bool 		GetWaterCO2(ERecalculate &recalcMode, ECO2Usage &CO2usage, char* taskName = "");
		
		virtual void		WriteAsHorzTable(int nos);
		virtual void		WriteAsHorzTable(ostream &fout);
		virtual void		WriteHeader(ostream &fout);
		
		virtual void		WriteTables();
		virtual void		WriteTable(int nos);
		virtual void		WriteTable(ostream &fout);
		
		virtual void		WriteAsVertTable(int nos, int numCols, bool printAll);
		virtual void		WriteAsVertTable(ofstream &fout, int numCols, bool printAll);	
		
		virtual void		GetTASName(ERecalculate recalcMode, ECO2Usage CO2usage);
		virtual void		GetTASName(char *fullName, char *rootName, char *fieldSymbol, 
											SOxidepc &ox, SNormpc &norm, 
											ERecalculate recalcMode, ECO2Usage CO2usage);

		virtual bool		IsPeralkaline(SOxidepc &ox);
		virtual bool		IsSodic(SOxidepc &ox);
		virtual bool		IsComenditic(char * str, SOxidepc &ox);
		virtual void		GetKState(char * str, SOxidepc &ox);
				
		virtual void		Recalculate(ERecalculate recalcMode);
		virtual void		Recalculate(SOxidepc &oxides, ERecalculate recalcMode);
		virtual double		GetAnalysisTotal();

		virtual bool		CalcCIPW(ECO2Usage CO2usage);
		virtual bool		CalcCIPW(ECO2Usage CO2usage, SOxidepc &oxides, SNormpc &norm);

		virtual bool		IsPositiveNumber(char *inStr);

		SSpecimenName		specimen;
		SOxidepc			oxide;
		SNormpc				norm;
		char				TASFieldSymbol[3];
		char				TASOptName[lenOptName];
		char				TASRootName[lenRootName];

	protected:
	
		SAnalysisData		data[numTableCols];
		int					columnsStored;
		int					tableSeq;
		int					multiColTableInUse;
		bool				printAllRows;
		
		char				textInput[lenTextInput];
		
		int					oxOrder[numOxides];
		bool				oxUsed[numOxides];

		EReadStatus			lastReadResult;		
		bool	 			validRow;

		int					inputRow;
		int					inputCol;

		ifstream 			inputFile;
		bool				inputDefined;
		bool				inputHeaderRead;
		SFileName			inputFileName;
		long int			inputRecRead;
		
		ofstream			outputFile[numOutputFiles];
		bool				outputDefined[numOutputFiles];
		bool				outputHeaderWritten[numOutputFiles];
		SFileName			outputFileName[numOutputFiles];
		long int			outputRecWritten[numOutputFiles];
		char				outputTableType[numOutputFiles];
		
		virtual void		ReadHeader(ifstream &input);
		
		virtual void		WriteVertTable(ofstream &fout);
		virtual void		Fudge(double &val);
		
		virtual bool		RowHasValues(int i, bool isOxide);
		
		virtual bool		AboveLine(double x, double y, SLine theLine);
		virtual bool		AboveLine(SLine theLine);

		virtual void		CreateAutoFileName(char* oldName, char* newName, char tableType);
		
		virtual void		AmendSpecimenName();
		virtual void		PlaceStrAndValue(ostream &fout, int initSpaces, char *inStr, int widStr, 
								double inValue, int widVal, bool printIt);

		virtual void		GetTabDelimitedStr(ifstream &input, char *inStr, int maxLen);
		virtual void		GetDouble(ifstream &input, double &f);
		
	private:	
	
		virtual void		CheckOutputFile(char *fName, int &nos, char *routineName);
		virtual bool		FileInUse(char* fName);
		
		virtual void		AbortDataError(char* inStr, char* mess);
		virtual void		AbortProgramingError(char* routine, char* mess1, char* mess2);
		virtual void		AbortProgramingError(char* routine, char* mess1, int n, char* mess2);
		virtual void		AbortProgramingError(char* routine, char* mess);
		virtual void		UnexpectedWriteError(char* routine, char* analName);
		virtual void		Abort();
		virtual void		Warning(char* mess1, char* mess2 = "", char* mess3= "", char* mess4= "");

};

#endif