// ===========================================================================
//
//	Tasks.cpp   			 Copyright R.W. Le Maitre 2002 - Version 1.0 PC
//
// ===========================================================================

#include "IUGSTAS.h"
#include "Tasks.h"

// 
//		 TASNames
// 
void TASNames()
{
	IUGSTAS			a;
	ERecalculate 	recalcMode;
	ECO2Usage 		CO2usage;
	char			taskName[20] = "TASNames()";
		
	do {
		if(!a.GetWaterCO2(recalcMode, CO2usage, taskName)) return;
		if(!a.AskForInputFile(taskName)) return;
		if(!a.AskForOutputTables(taskName)) return;

		do {
			a.Init();
			a.ReadTabDelimitedRow();
			if(a.EndOfFile()) break;	
			a.GetTASName(recalcMode, CO2usage);
			a.WriteTables();
		} while(!UserAbort());

		a.CloseInputFile();
		a.CloseAllOutputFiles();
	} while(true);
}

// 
//		 TASNameInteractive
// 
void TASNameInteractive()
{
	ERecalculate 	recalcMode;
	ECO2Usage 		CO2usage;
	
	IUGSTAS		c;
	
	do {
		if(!c.GetWaterCO2(recalcMode, CO2usage, "TASNameInteractive()")) return;
		c.GetOxidesInterActively();

		c.GetTASName(recalcMode, CO2usage);
		c.WriteTable(cout);
		
		cout << endl;
	} while (true);
}


#pragma mark -
// 
//		 TASNamesTest
// 
void TASNamesTest()
{
	IUGSTAS		obj1;									// line 1
	
	obj1.OpenInputFile("TestTAS");						// line 2
	obj1.OpenOutputFile("Table1", 1);				 	// line 3
	obj1.OpenOutputFile("Table2", 2);					// line 4
	obj1.OpenOutputFile("Table3", 3);					// line 5
		
	do {												// line 6
		obj1.Init();									// line 7
		obj1.ReadTabDelimitedRow();						// line 8
		if(obj1.EndOfFile()) break;						// line 9
		obj1.GetTASName(kWaterCO2Free, kCO2AsNaCarb);	// line 10
		obj1.WriteTable(1);								// line 11
		obj1.WriteAsVertTable(2, 7, false);				// line 12
		obj1.WriteAsHorzTable(3);						// line 13
	} while(true);										// line 14
 	
	obj1.CloseInputFile();								// line 15
	obj1.CloseOutputFile(1);							// line 16
	obj1.CloseOutputFile(2);							// line 17
	obj1.CloseOutputFile(3);							// line 18
}

// 
//		 RandomCIPWTest					
// 
void RandomCIPWTest(int maxBadTotals)
{
	IUGSTAS		a;
	int 		badTotals = 0, numMajors = 10;
	long int	tries = 0;
	
	srand((unsigned int)time(NULL));	// initialise random number generator

	a.OpenOutputFile("test", 1);

	a.Init(); 

	a.oxide.named.SiO2 = 50.0;
	a.oxide.named.TiO2 = 5.0;
	a.oxide.named.Al2O3 = 15.0;
	a.oxide.named.Fe2O3 = 3.0;
	a.oxide.named.FeO = 5.0;
	a.oxide.named.MnO = 5.0;
	a.oxide.named.MgO = 10.0;
	a.oxide.named.CaO = 4.0;
	a.oxide.named.Na2O = 2.0;
	a.oxide.named.K2O = 1.0;
	
	SOxidepc temp = a.oxide;		// store the analysis

	do {
		a.Init(); 
		// create a new analysis with values placed randomly from the stored data.
		for(int j = 0; j < numMajors; j++) {
			double val = temp.wt[j];
			if(val > 0.0) {
				do {
					int k = (int)fmod(double(rand()), double(numOxides - 2));
					if(k < 0 || k > numOxides) {
						cout << k;
						exit(1);
					}
					if(a.oxide.wt[k] == 0.0) {
						a.oxide.wt[k] = val;
						break;
					}
				}while(true); 
			} else {
				cout << "Val = 0" << endl;
			}
		}
		
		if(!a.CalcCIPW(kCO2AsCaMgFeCarb)) {
			a.WriteTable(1);
			badTotals++;
			if(badTotals > maxBadTotals) break;
		}
		tries++;

		if(fmod((double)tries, 200.0) == 0) {
			cout << tries << " CIPW calculations done - abort with " << AbortCmd() << endl;
		}

	} while (!UserAbort()); 
	
	a.CloseOutputFile(1);
	
	cout << endl << endl;
	cout << "Total of " << tries << " CIPW calculations done. " << endl;
	
	if(badTotals > 0) {
		cout << badTotals << " analyses with bad totals.";
	} else {
		cout << "No analyses with bad totals.";
	}
	cout << endl << endl;
}


// ===========================================================================
// 	Machine dependent code
//
// 	The following code is not standard C++ and has been specifically written for 
//	IBM PC 
// ===========================================================================
#include <conio.h>
// 
//		 UserAbort			
// 
// It returns true if the user has typed Cmd + period otherwise its returns false
bool UserAbort()
{
  bool  userAborting = false;
  if (kbhit())  // check if a key has been pressed
      {
       getch();  // if a key has been hit then remove it from the queue.
       userAborting = true;
      }
	return userAborting;
}

// 
//		 AbortCmd			
// 
char* AbortCmd()
{
	return "any key";
}
